import os
import math
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib import cm

# Function to read data from a file
def read_data(file_path):
    data = pd.read_csv(file_path, sep='\s+', header=None, names=['Frequency', 'SPL'])
    return data

# Function to extract angle from filename
def extract_angle(file_name):
    angle_str = file_name.split('_')[-1].split('.')[0]  # Get the angle part from the filename
    return int(angle_str)

EQ = ""
colorscale = 'nipy_spectral'
# colorscale = 'turbo'
# /!\ -------EQ option-----------
EQoption = 'Norm' # Can be 'without', 'EQed', 'Norm'
fc=0. ; frear=0

# Directory containing the text files ; plot title
# directory = './PWD3_DAEX25'; title = 'Plywood 3mm H25xV125cm test panel (raw), V axis=grain, H directivity, DAEX25FHE, @1m '; frear=2500; fc=12500
# directory = './EPStest_directivity'; title = 'EPS 10mm H30xV40cm test panel (1 side PVA), H directivity, DAEX25FHE, @1m'
# directory = './FRS8_directivity'; title = 'Full range Visaton FRS8 in 1.2l closed box, @1m'
# directory = './PStest_CT13'; title = 'PS 1mm H30xV40cm test panel, H directivity, CT13, @1m'
#directory = './PStest_DAEX25'; title = 'PS 1mm H30xV40cm test panel, H directivity, DAEX25FHE, @1m'
# directory = './PWD3panel_DAEX25'; title = 'Plywood 3mm H42xV120cm panel (varnished), V axis=grain, H directivity, DAEX25FHE, @1m'; frear=1900; fc=12500
# directory = './PWD5_across_grain_DAEX25'; title = 'Plywood 5mm H34xV42cm test panel(raw), V axis=grain, H directivity, DAEX25FHE, @1m'; fc=8000.
# directory = './PWD5_along_grain_DAEX25'; title = 'Plywood 5mm H34xV42cm test panel(raw), V axis=grain, V directivity, DAEX25FHE, @1m'
directory = './PWD3_DAEX25_on_piles'; title = 'Plywood 3mm H25xV125cm test panel (raw), V axis=grain, H dir., DAEX25FHE "on piles", @1m '; fc=12500.; frear=3300.

# Initialize lists to store frequency, angle, and SPL data
frequencies = []
angles_raw = []
spl_values = []

# Read all text files in the directory
for file_name in os.listdir(directory):
    if file_name.endswith('.txt'):
        file_path = os.path.join(directory, file_name)
        data = read_data(file_path)
        print(file_name)       
        # Extract angle and append data
        angle = extract_angle(file_name)
        # print(angle)
        frequencies.append(data['Frequency'].values)
        spl_values.append(data['SPL'].values)
        angles_raw.append(angle)

# Convert lists to numpy arrays
frequencies = np.unique(np.concatenate(frequencies))
# print(frequencies)
angles = np.sort(np.unique(angles_raw))
# print(angles_raw)
# print(angles)
spl_values = np.array(spl_values)
# spl_values = spl_values - np.mean(spl_values) + 80. # set mean to 80dB
# print(spl_values.ndim)


# Create a meshgrid for frequencies and angles
F, A = np.meshgrid(frequencies, angles)

# Create a grid
SPL_grid = np.zeros((len(angles), len(frequencies)))

for i, angle in enumerate(angles):
    # Find the index of the angle in the angles list
    index = np.where(angles_raw == angle)[0][0]
    # print(i, angle, index) 
    SPL_grid[i, :] = spl_values[index]
    # SPL_grid[i, :] = np.interp(frequencies, frequencies[index], spl_values[index])

# Adjust gain to have a 80dB mean value in the -20° to 20° range
gain = 80 - np.mean(SPL_grid[7:10,:])
SPL_grid[:,:] = SPL_grid[:,:] + gain

if EQoption == 'EQed':
    EQ=' EQed'
    for j, freq in enumerate(frequencies):
        gain = 80 - np.mean(SPL_grid[7:12,j])
        SPL_grid[:,j] = SPL_grid[:,j] + gain

if EQoption == 'Norm':
    EQ=' Norm'
    for j, freq in enumerate(frequencies):
        gain = 80 - np.mean(SPL_grid[9,j])
        SPL_grid[:,j] = SPL_grid[:,j] + gain

# Define the levels for the contour plot
levels = np.linspace(50, 99.5, 100)  # 100 levels from 50 dB to 100 dB
#print(levels)



# Plotting the spinorama
plt.figure(figsize=(10, 6))
plt.contourf(F, A, SPL_grid, levels=levels, cmap=colorscale, vmin=50., vmax=100, extend='both')
plt.colorbar(label='SPL (dB)')
plt.title(title + EQ)
plt.xlabel('Frequency (Hz)')
plt.ylabel('Angle (degrees)')
plt.xscale('log')  # Optional: log scale for frequency
plt.grid()
if EQoption != 'none':
    if fc != 0:
        for alpha in range(-90, 0, 5):
            f = fc / math.sin(alpha*math.pi/180)**2
            if f < 20000:
                plt.plot(f,alpha,'k+')
        for alpha in range(90, 0, -5):
            f = fc / math.sin(alpha*math.pi/180)**2
            if f < 20000:
                plt.plot(f,alpha,'k+')
        plt.plot([fc,fc], [-80,80], color='black', linestyle='dotted')
    if frear != 0:
        plt.plot([frear,frear], [-90,90], color='black', linestyle='dotted')
        # plt.plot(frear,0,'ko')
plt.show()

